﻿using UnityEngine;
using System.Collections;

public class LevelManager : MonoBehaviour {

	private static LevelManager _instance;
	public static LevelManager Instance {
		get
		{
			if (!_instance)
			{
				_instance = GameObject.FindObjectOfType<LevelManager>();
				if (!_instance)
				{
					GameObject container = new GameObject();
					container.name = "LevelManager";
					_instance = container.AddComponent<LevelManager>();
				}
			}
			
			return _instance;
		}
	}

	public Camera gameCamera;

	private Prefabs prefabs;

	private float viewportWidth;
	private float viewportHeight;

	private Transform dirtParent;

	private float lastRowY;

	private float rockPercentage = 0.05f;

	private float startPercentage = 0.5f;
	private float screenPercentage = 0.33f;

	// Use this for initialization
	void Start () {
		prefabs = Prefabs.Instance;

		if (gameCamera == null)
			gameCamera = Camera.main;

		viewportHeight = gameCamera.orthographicSize * 2.0f;
		viewportWidth = viewportHeight * gameCamera.aspect;

		dirtParent = new GameObject("Dirt").transform;

		for (float y = 0; y > gameCamera.transform.position.y - (viewportHeight*startPercentage) - 1.0f; y -= 1.0f)
		{
			for (float x = -viewportWidth/2.0f; x < viewportWidth/2.0f; x += 1.0f)
			{
				(GameObject.Instantiate(prefabs.dirtPrefab, new Vector3(x, y, 0f), Quaternion.identity) as GameObject).transform.parent = dirtParent;
			}
		}

		lastRowY = gameCamera.transform.position.y - (viewportHeight*startPercentage);
	}
	
	public void MoveCamera(float lowestRoot) {
		if (gameCamera.transform.position.y - (viewportHeight*(startPercentage-screenPercentage)) < lowestRoot)
			return;

		float newY = lowestRoot + (viewportHeight*(startPercentage-screenPercentage));

		gameCamera.transform.position = new Vector3(gameCamera.transform.position.x, newY, gameCamera.transform.position.z);

		if (gameCamera.transform.position.y - (viewportHeight*startPercentage) <= lastRowY)
		{
			lastRowY -= 1.0f;

			bool placedRock = false;
			float offset = Random.Range (-0.5f, 0.5f);

			for (float x = -viewportWidth/2.0f; x < viewportWidth/2.0f; x += 1.0f)
			{
				(GameObject.Instantiate(prefabs.dirtPrefab, new Vector3(x, lastRowY, 0f), Quaternion.identity) as GameObject).transform.parent = dirtParent;

				float rand = Random.Range(0f, 1f);

				if (!placedRock && rand < rockPercentage)
				{
					(GameObject.Instantiate(prefabs.GetRockPrefab(), new Vector3(x + offset, lastRowY, 0f), Quaternion.identity) as GameObject).transform.parent = dirtParent;
					placedRock = true;
				}
				else
				{
					placedRock = false;
				}
			}
		}

		gameCamera.transform.position = new Vector3(gameCamera.transform.position.x, newY, gameCamera.transform.position.z);
	}
}
