﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class Root : MonoBehaviour {

	private Direction _currentDirection = Direction.DOWN;
	public Direction CurrentDirection {
		get { return _currentDirection; }
		set { _currentDirection = value; }
	}

	private Vector2 _position;
	public Vector2 Position {
		get { return _position; }
		set { _position = value; }
	}

	private int _widthPixels = 20;
	public int WidthPixels {
		get { return _widthPixels; }
		set { _widthPixels = value; }
	}

	private int _heightPixels = 20;
	public int HeightPixels {
		get { return _heightPixels; }
		set { _heightPixels = value; }
	}

	private float widthUnits;
	private float heightUnits;

	private GameObject container;
	private GameObject rootTip;
	private List<GameObject> rootPieces;

	private Collider2D[] collisionResults;

	private GameManager gameManager;
	private Prefabs prefabs;

	private bool selected = false;
	
	public void Instantiate (GameObject gameObject, Vector2 startPosition, Direction startDirection) {
		container = gameObject;
		_position = startPosition;
		_currentDirection = startDirection;

		widthUnits = _widthPixels / (float)Constants.PIXELS_PER_UNIT;
		heightUnits = _heightPixels / (float)Constants.PIXELS_PER_UNIT;

		gameManager = GameManager.Instance;
		prefabs = Prefabs.Instance;

		rootTip = GameObject.Instantiate(prefabs.rootTipPrefab, _position, Quaternion.identity) as GameObject;
		rootTip.transform.parent = container.transform;
		rootPieces = new List<GameObject>();
		rootPieces.Add (rootTip);

		collisionResults = new Collider2D[2];

		StartCoroutine(SetDirectionDown());
	}

	public void Move() {
		GameObject rootPiece = GameObject.Instantiate(prefabs.rootPrefab, _position, GetSpriteRotation()) as GameObject;
		rootPiece.transform.parent = container.transform;
		rootPiece.transform.localScale = GetSpriteScale();

		if (selected)
		{
			SpriteRenderer[] childSprites = rootPiece.GetComponentsInChildren<SpriteRenderer>();
			
			for (int j = 0; j < childSprites.Length; j++)
			{
				if (childSprites[j].gameObject.layer == Constants.HIGHLIGHT_LAYER)
					childSprites[j].enabled = true;
			}
		}

		rootPieces.Add (rootPiece);

		UpdatePosition();

		rootTip.transform.position = _position;
		rootTip.transform.rotation = GetSpriteRotation();
		rootTip.transform.localScale = GetSpriteScale();

		if (DidCollide())
		{
			gameManager.RemoveRoot(this);
		}
	}

	public void Select() {
		for (int i = 0; i < rootPieces.Count; i++)
		{
			SpriteRenderer[] childSprites = rootPieces[i].GetComponentsInChildren<SpriteRenderer>();

			for (int j = 0; j < childSprites.Length; j++)
			{
				if (childSprites[j].gameObject.layer == Constants.HIGHLIGHT_LAYER)
					childSprites[j].enabled = true;
			}
		}

		selected = true;
	}

	public void Deselect() {
		for (int i = 0; i < rootPieces.Count; i++)
		{
			SpriteRenderer[] childSprites = rootPieces[i].GetComponentsInChildren<SpriteRenderer>();
			
			for (int j = 0; j < childSprites.Length; j++)
			{
				if (childSprites[j].gameObject.layer == Constants.HIGHLIGHT_LAYER)
					childSprites[j].enabled = false;
			}
		}

		selected = false;
	}

	private IEnumerator SetDirectionDown() {
		yield return new WaitForSeconds(Constants.MOVEMENT_RATE*4f);

		_currentDirection = Direction.DOWN;
	}

	private Quaternion GetSpriteRotation() {
		switch (_currentDirection)
		{
		case Direction.LEFT:
			return new Quaternion(0f, 0f, -0.7f, 0.7f);
		case Direction.RIGHT:
			return new Quaternion(0f, 0f, 0.7f, 0.7f);
		}

		return Quaternion.identity;
	}

	private Vector3 GetSpriteScale() {
		switch (_currentDirection)
		{
		case Direction.LEFT:
			return Vector3.one;
		case Direction.RIGHT:
			return new Vector3(-1f, 1f, 1f);
		}
		
		return Vector3.one;
	}

	private void UpdatePosition() {
		switch (_currentDirection)
		{
		case Direction.LEFT:
			_position.x -= widthUnits;
			break;
		case Direction.RIGHT:
			_position.x += widthUnits;
			break;
		case Direction.DOWN:
			_position.y -= heightUnits;
			break;
		}
	}

	private bool DidCollide() {
		Vector2 center = _position;

		switch (_currentDirection)
		{
		case Direction.LEFT:
			center.x -= widthUnits/2.0f;
			break;
		case Direction.RIGHT:
			center.x += widthUnits/2.0f;
			break;
		case Direction.DOWN:
			center.y -= heightUnits/2.0f;
			break;
		}

		return (Physics2D.OverlapPointNonAlloc(center, collisionResults, (1 << Constants.ROOT_LAYER) | (1 << Constants.BOUNDARY_LAYER) | (1 << Constants.OBSTACLE_LAYER)) > 1);
	}
}
